/**
 * MProperParetoOptimality.java
 * 
 * @author Marlon A. Braun
 * @version 1.0
 */
package jmetal.util;

import jmetal.base.Solution;
import jmetal.base.SolutionSet;
import jmetal.base.operator.comparator.ObjectiveComparator;
import java.util.*;

/**
 * This class creates a M-Proper Pareto-optimal <code>SolutionSet</code> from a non-dominated <code>SolutionSet</code>.
 * It also returns the solutions, which are M-proper Pareto-dominated. 
 * @author Marlon
 *
 */
public class MProperParetoOptimality {
	
	/**
	 * Number, which is used to test M-proper Pareto-optimality 
	 */
	private double M;
	
	/**
	 * Constructor
	 * @param M Number which is used for the MPPO sorting
	 */
	public MProperParetoOptimality(double M){
		this.M = M;
	}
	
	/**
	 * This method splits a Pareto-optimal <code>SolutionSet</code> into a M-Proper dominant and dominated <code>SolutionSet</code>.
	 * @param solutionSet The SolutionSet, which is to be split
	 * @return The split SolutionSet
	 */
	public SolutionSet[] create(SolutionSet solutionSet){
		//SplitSet[0] stores M-Proper Pareto-optimal SolutionSet
		//SplitSet[1] stores the dominated solutions
		//STORING SOLUTION IN LISTS AND TRANSFORMING THEM AFTRERWARDS TO SOLUTIONSETS COULD IMPROVE STORAGE USAGE
		SolutionSet[] SplitSet = new SolutionSet[2];
		SplitSet[0] = new SolutionSet(solutionSet.size());
		SplitSet[1] = new SolutionSet(solutionSet.size());
		
		//Iterate through the SolutionSet and check for each solution MPPO
		Iterator it = solutionSet.iterator();
		while(it.hasNext()){
			Solution currentSolution = (Solution) it.next();
			
			boolean flag = true;
			Iterator It_innerLoop = solutionSet.iterator();
			
			//Check each solutions with each other solution form the SolutionSet
			//Flag remains true, as long as the test criterion ist fullfilled
			//Set flag to false, if there ist one objective, for which there is no other objective, which fullfills the test
			while(It_innerLoop.hasNext() && flag == true){
				Solution sol = (Solution) It_innerLoop.next();
				//Do the test by the inequality formula
				flag = isMProperParetoOptimal(currentSolution, sol);
			}
			
			if(flag == true){
				SplitSet[0].add(currentSolution);
			} else {
				SplitSet[1].add(currentSolution);
			}
		}
		
		return SplitSet;
	}
	
	/**
	 * This method compares two solutions for MPPO
	 * @param sol1 Solution to be checked for dominance
	 * @param sol2 Solution, which is used for checking
	 * @return Returns true, if sol1 is MPPO w.r.t. sol2
	 */
	public boolean isMProperParetoOptimal(Solution sol1, Solution sol2){
		boolean flag = true;
		
		//Store which solution is dominant for which objective
		LinkedList<Integer> sol1Dominate = new LinkedList<Integer>(); //j
		LinkedList<Integer> sol2Dominate = new LinkedList<Integer>(); //i
		
		for(int i = 0 ; i < sol1.numberOfObjectives() ; i++){
			
			//Use ObjectiveComparator from jmetal.base.operator.comparator.ObjectiveComparator
			ObjectiveComparator comp = new ObjectiveComparator(i);
			int res = comp.compare(sol1, sol2);
			
			if(res == -1){
				sol1Dominate.add(i);
			} else if(res == 1){
				sol2Dominate.add(i);
			}
			
		}
		
		//Do the test
		//Flag remains false, as long as the proper Pareto optimality condition is not satisfied
		//Check for each objective, for which sol2 is better, if there is an objective of sol1, which satisfies the condition
		Iterator ItSol2Dom = sol2Dominate.iterator();
		while(ItSol2Dom.hasNext()){
			flag = false;
			Iterator ItSol1Dom = sol1Dominate.iterator();
			int i = (Integer) ItSol2Dom.next();
			double f_i_x1 = sol1.getObjective( i );
			double f_i_x2 = sol2.getObjective( i );
			while(ItSol1Dom.hasNext() && flag == false){
				//Check for each objective of sol1, for which sol1 is dominant
				int j = (Integer) ItSol1Dom.next();
				double f_j_x1 = sol1.getObjective( j );
				double f_j_x2 = sol2.getObjective( j );
				//Using the actual formula
				//If inequality is satisfied, the inner loop stops, the next objective from the outer loop is tested
				if( (f_i_x1 - f_i_x2)/(f_j_x2 - f_j_x1)  <= M){
					flag = true;
				}
			}//while
			//If no objective value was found, which satifies the condition, MPPO is not satisfied
			if(flag == false){
				return false;
			}
		}//while
		
		return true;
	}

}
