package jmetal.util.io;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.StringTokenizer;

import jmetal.base.Solution;
import jmetal.base.SolutionSet;
import jmetal.util.MProperParetoOptimality;

/**
 * This class converts Pareto front files to M Proper Pareto front files
 * @author Marlon A. Braun
 *
 */
public class ParetoToM {
	
	private String source;
	private String target;
	private double m;
	
	/**
	 * Creates an instance of PareToM, which may then be used to either convert
	 * one file or a complete file system.
	 * @param source	source path of the filesystem / path of file which is to be converted
	 * @param target	target path, where converted file is to be saved
	 * @param m			M values used for doing M Proper sorting
	 */
	public ParetoToM(String source, String target, double m){
		this.source = source;
		this.target = target;
		this.m = m;
	}
	
	/**
	 * Converts a single file
	 * @throws FileNotFoundException
	 */
	public void convertFile() throws FileNotFoundException{
		//Pathname of source and destination, value of M used for MPPO

		String sourcepath = source;
		
		Double m_aux = new Double(m);
		String m_string = m_aux.toString();
		String destinationpath = target + "M" + m_string; 
		
		File source = new File(sourcepath);
		File destination = new File(destinationpath);
		
		try {
			// Initializes the input reader
			FileInputStream fis = new FileInputStream(source);
			InputStreamReader isr = new InputStreamReader(fis);
			BufferedReader br = new BufferedReader(isr);
			
			// Read first line, initialize list of solutions and store number of objectives
			StringTokenizer st = new StringTokenizer(br.readLine());
			int nObj = st.countTokens();
			
			double currentValue;
			int j = 0;
			
			LinkedList<Solution> list = new LinkedList<Solution>();
			Solution sol = new Solution(nObj);
			
			while(st.hasMoreTokens()){
				currentValue = Double.valueOf(st.nextToken()).doubleValue();
				sol.setObjective(j, currentValue);
				j++;
			}

			list.add(sol);
			
			// Read solutions line by line	
			while(br.ready()){
				st = new StringTokenizer(br.readLine());
				sol = new Solution(nObj);
				
				j = 0;
				while(st.hasMoreTokens()){
					currentValue = Double.valueOf(st.nextToken()).doubleValue();
					sol.setObjective(j, currentValue);
					j++;
				}
				
				list.add(sol);
			}
			
			SolutionSet solutionset = new SolutionSet( list.size() );
			
			// Write solutions in list to SolutionSet
			Iterator it = list.iterator();
			while(it.hasNext()){
				sol = (Solution) it.next();
				solutionset.add(sol);
			}
			
			//Remove duplicates
			solutionset.removeDuplicates();
			
			// Do MPPO Sorting
			MProperParetoOptimality mppo = new MProperParetoOptimality(m);
			SolutionSet results[] = mppo.create(solutionset);
			
			br.close();
			
			try{
				// Initialize output writer
				FileOutputStream fos   = new FileOutputStream(destination);
			    OutputStreamWriter osw = new OutputStreamWriter(fos);
			    BufferedWriter bw      = new BufferedWriter(osw);
			    for (int i = 0; i < results[0].size(); i++) {
			        bw.write(results[0].get(i).toString());
			        bw.newLine();
			    }
			    
			    bw.close();
			} catch(IOException e){
				System.out.println("Writing MPPO Set failed.");
			}
			
		} catch (IOException e){
			System.out.println("Source file could not be found.");
		}
	}
	
	/**
	 * Converts a FileSystem 
	 * @throws FileNotFoundException
	 */
	public void convertFileSystem() throws FileNotFoundException{
		
		//Pathname of source and destination, value of M used for MPPO
		File folder = new File(source);
		String[] files = folder.list();
		
		for(int k = 0 ; k < files.length ; k++){
		
			String filename = files[k];
			System.out.println("Converting: " + files[k]);
		
			String sourcepath = source + "/" + filename;
			Double m_aux = new Double(m);
			String m_string = m_aux.toString();
			String destinationpath = target + "/" + "M" + m_string + "." + filename; 
		
			File source = new File(sourcepath);
			File destination = new File(destinationpath);
		
			try {
				// Initializes the input reader
				FileInputStream fis = new FileInputStream(source);
				InputStreamReader isr = new InputStreamReader(fis);
				BufferedReader br = new BufferedReader(isr);
			
				// Read first line, initialize list of solutions and store number of objectives
				StringTokenizer st = new StringTokenizer(br.readLine());
				int nObj = st.countTokens();
				
				double currentValue;
				int j = 0;
				
				LinkedList<Solution> list = new LinkedList<Solution>();
				Solution sol = new Solution(nObj);
				
				while(st.hasMoreTokens()){
					currentValue = Double.valueOf(st.nextToken()).doubleValue();
					sol.setObjective(j, currentValue);
					j++;
				}

				list.add(sol);
				
				// Read solutions line by line	
				while(br.ready()){
					st = new StringTokenizer(br.readLine());
					sol = new Solution(nObj);
					
					j = 0;
					while(st.hasMoreTokens()){
						currentValue = Double.valueOf(st.nextToken()).doubleValue();
						sol.setObjective(j, currentValue);
						j++;
					}
					
					list.add(sol);
				}
			
				SolutionSet solutionset = new SolutionSet( list.size() );
			
				// Write solutions in list to SolutionSet
				Iterator it = list.iterator();
				while(it.hasNext()){
					sol = (Solution) it.next();
					solutionset.add(sol);
				}
				
				//Remove duplicates
				solutionset.removeDuplicates();
			
				// Do MPPO Sorting
				MProperParetoOptimality mppo = new MProperParetoOptimality(m);
				SolutionSet results[] = mppo.create(solutionset);
			
				br.close();
			
				try{
					// Initialize output writer
					FileOutputStream fos   = new FileOutputStream(destination);
					OutputStreamWriter osw = new OutputStreamWriter(fos);
					BufferedWriter bw      = new BufferedWriter(osw);
					for (int i = 0; i < results[0].size(); i++) {
						bw.write(results[0].get(i).toString());
						bw.newLine();
					}
			    
					bw.close();
				} catch(IOException e){
					System.out.println("Writing MPPO Set failed.");
				}
			
			} catch (IOException e){
				System.out.println("Source file could not be found.");
			}
		}
		
	}

}
